package io.data2viz.charts.logging

import io.data2viz.charts.core.measureText
import io.data2viz.charts.layout.DrawingZone
import io.data2viz.color.col
import io.data2viz.geom.Rect
import io.data2viz.math.pct
import kotlin.math.pow
import kotlin.math.sqrt

internal inline fun DrawingZone.updateWatermarkNode() {
    val fSize = contentSize
    clear()
    group {
        transform { rotate(-kotlin.math.PI / 6) }
        var chartsTxtBounds: Rect? = null
        var freeVersionsBounds: Rect? = null
        val data2vizColor = "#6047de".col.withAlpha(alpha = 25.pct)
        val waterMarkTextColor = "#000000".col.withAlpha(alpha = 15.pct)
        val textSize = 36.0
        val maxY = sqrt(fSize.width.pow(2) + fSize.height.pow(2))
        var yOffset = .0
        var i = 0
        do {
            var xOffset = -(fSize.width * 1.5)
            val dynamicXOffset = 0
            do {
                repeat(2) { j ->
                    when {
                        ((j == 0) && ((i % 2) == 0)) || ((j == 1) && ((i % 2) == 1)) -> text {
                            textContent = "charts.kt"
                            hAlign = io.data2viz.viz.TextHAlign.LEFT
                            vAlign = io.data2viz.viz.TextVAlign.BASELINE
                            fontFamily = io.data2viz.viz.FontFamily.SANS_SERIF
                            fontSize = textSize
                            textColor = data2vizColor
                            val bounds = chartsTxtBounds ?: measureText().also { chartsTxtBounds = it }
                            x = xOffset - dynamicXOffset
                            xOffset += bounds.width
                            y = yOffset
                        }

                        else -> text {
                            textContent = "    free version    "
                            hAlign = io.data2viz.viz.TextHAlign.LEFT
                            vAlign = io.data2viz.viz.TextVAlign.BASELINE
                            fontFamily = io.data2viz.viz.FontFamily.SANS_SERIF
                            fontSize = textSize
                            textColor = waterMarkTextColor
                            val bounds = freeVersionsBounds ?: measureText().also { freeVersionsBounds = it }
                            x = xOffset - dynamicXOffset
                            xOffset += bounds.width
                            y = yOffset
                        }
                    }
                }
            } while (xOffset < fSize.width)
            yOffset += chartsTxtBounds!!.height * 2.5
            i++
        } while (yOffset < maxY)
    }
}
