package io.data2viz.charts.chart

import io.data2viz.charts.core.Datum
import io.data2viz.charts.dimension.Constant
import io.data2viz.charts.dimension.Discrete
import io.data2viz.charts.dimension.Quantitative
import io.data2viz.charts.dimension.Temporal
import kotlinx.datetime.Instant
import kotlinx.datetime.LocalDateTime


/**
 * Create a [Quantitative] dimension for the [Chart].
 *
 * @param accessor The "accessor" is a lambda used to retrieve a [Double]? from a [Datum].
 * @param init The initialization lambda that will be applied to the [Dimension] just after its instantiation.
 */
public fun <DOMAIN> Chart<DOMAIN>.quantitative(
    accessor: Datum<DOMAIN>.() -> Double?,
    init: Quantitative<DOMAIN>.() -> Unit = {}
):
    Quantitative<DOMAIN> = Quantitative(accessor).apply(init)


/**
 * Create a [Discrete] dimension for the [Chart].
 *
 * @param accessor The "accessor" is a lambda used to retrieve a value from a [Datum].
 * @param init The initialization lambda that will be applied to the [Dimension] just after its instantiation.
 */
public fun <DOMAIN, VALUE> Chart<DOMAIN>.discrete(
    accessor: Datum<DOMAIN>.() -> VALUE,
    init: Discrete<DOMAIN, VALUE>.() -> Unit = {}
):
    Discrete<DOMAIN, VALUE> = Discrete(accessor).apply(init)

/**
 * Create a [Constant] dimension for the [Chart].
 *
 * @param constantValue The value this [Dimension] will always have.
 * @param init The initialization lambda that will be applied to the [Dimension] just after its instantiation.
 */
public fun <DOMAIN, VALUE> Chart<DOMAIN>.constant(constantValue: VALUE, init: Constant<DOMAIN, VALUE>.() -> Unit = {}):
    Constant<DOMAIN, VALUE> = Constant<DOMAIN, VALUE>(constantValue).apply(init)

/**
 * Create a [Temporal] dimension for the [Chart].
 *
 * @param accessor The "accessor" is a lambda used to retrieve a [LocalDateTime] from a [Datum].
 * @param init The initialization lambda that will be applied to the [Dimension] just after its instantiation.
 */
public fun <DOMAIN> Chart<DOMAIN>.temporal(
    accessor: Datum<DOMAIN>.() -> Instant,
    init: Temporal<DOMAIN>.() -> Unit = {}
):
    Temporal<DOMAIN> = Temporal(accessor).apply(init)
