package io.data2viz.charts.chart.mark

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.mark.internal.DotMarkImpl
import io.data2viz.charts.config.AxisConfig
import io.data2viz.charts.config.EventConfig
import io.data2viz.charts.config.MarkConfig
import io.data2viz.charts.core.Dataset
import io.data2viz.charts.dimension.Dimension

/**
 * Add a [DotMarkImpl] to the current [Chart].
 *
 * See [Scatter plot](https://en.wikipedia.org/wiki/Scatter_plot) for more info.
 *
 * @param x The [Dimension] for the X axis, this dimension can be continuous or discrete
 * @param y The [Dimension] for the Y axis, this dimension can be continuous or discrete
 * @param init The initialization lambda for this [Mark]
 */
public fun <DOMAIN, XVAL, YVAL> Chart<DOMAIN>.dot(
    x: Dimension<DOMAIN, XVAL>,
    y: Dimension<DOMAIN, YVAL>,
    init: DotMark<DOMAIN, XVAL, YVAL>.() -> Unit = {}
): Unit = addMark { markIndex -> dotMark(markIndex, dataset, x, y, init, config.events, config.mark, config.x, config.y) }

@Deprecated("Use dot instead.", ReplaceWith("dot(x, y, init)"))
public fun <DOMAIN, XVAL, YVAL> Chart<DOMAIN>.plot(
    x: Dimension<DOMAIN, XVAL>,
    y: Dimension<DOMAIN, YVAL>,
    init: DotMark<DOMAIN, XVAL, YVAL>.() -> Unit = {}
): Unit = addMark { markIndex -> plotMark(markIndex, dataset, x, y, init, config.events, config.mark, config.x, config.y) }

/**
 * Instantiate a new [PlotMark].
 *
 * See [Scatter plot](https://en.wikipedia.org/wiki/Scatter_plot) for more info.
 *
 * @param dataset Your chart's [Dataset]
 * @param x The [Dimension] for the X axis, this dimension can be continuous or discrete
 * @param y The [Dimension] for the Y axis, this dimension can be continuous or discrete
 * @param init The initialization lambda for this [Mark]
 * @param config The [MarkConfig] to configure this [Mark]
 * @param xAxisConfig The [AxisConfig] to configure this mark's X axis
 * @param yAxisConfig The [AxisConfig] to configure this mark's Y axis
 */
public fun <DOMAIN, XVAL, YVAL> dotMark(
    markIndex: Int,
    dataset: Dataset<DOMAIN>,
    x: Dimension<DOMAIN, XVAL>,
    y: Dimension<DOMAIN, YVAL>,
    init: DotMark<DOMAIN, XVAL, YVAL>.() -> Unit,
    eventConfig: EventConfig,
    config: MarkConfig,
    xAxisConfig: AxisConfig,
    yAxisConfig: AxisConfig,
    useBandwidthForX: Boolean = false,
    useBandwidthForY: Boolean = false
): DotMark<DOMAIN, XVAL, YVAL> = DotMarkImpl(markIndex, dataset, x, y, init, eventConfig, config, xAxisConfig, yAxisConfig, useBandwidthForX, useBandwidthForY)

@Deprecated("Use dotMark instead.", ReplaceWith("dotMark(dataset, x, y, init, markConfig, xAxisConfig, yAxisConfig)"))
public fun <DOMAIN, XVAL, YVAL> plotMark(
    markIndex: Int,
    dataset: Dataset<DOMAIN>,
    x: Dimension<DOMAIN, XVAL>,
    y: Dimension<DOMAIN, YVAL>,
    init: DotMark<DOMAIN, XVAL, YVAL>.() -> Unit,
    eventConfig: EventConfig,
    config: MarkConfig,
    xAxisConfig: AxisConfig,
    yAxisConfig: AxisConfig
): DotMark<DOMAIN, XVAL, YVAL> = dotMark(markIndex, dataset, x, y, init, eventConfig, config, xAxisConfig, yAxisConfig)

/**
 * Draw signs (symbols) for each elements of a given [Dataset] and X and Y [Dimension]s.
 */
public interface DotMark<DOMAIN, XVAL, YVAL> :
    HasMarker<DOMAIN>,
    HasSize<DOMAIN>,
    HasStrokeColor<DOMAIN>,
    HasMarkDecorator<DOMAIN>,
    HasHighlightDecorator<DOMAIN>,
    HasSelectionDecorator<DOMAIN>,
    HasEventZones<DOMAIN>,
    HasLegendDecorator<DOMAIN>,
    HasHighlightMode,
    HasSelectionMode,
    MarkXY<DOMAIN, XVAL, YVAL>

@Deprecated("Use DotMark instead.")
public interface PlotMark<DOMAIN, XVAL, YVAL> :
    HasMarker<DOMAIN>,
    HasSize<DOMAIN>,
    HasStrokeColor<DOMAIN>,
    HasMarkDecorator<DOMAIN>,
    HasHighlightDecorator<DOMAIN>,
    HasSelectionDecorator<DOMAIN>,
    HasLegendDecorator<DOMAIN>,
    HasHighlightMode,
    HasSelectionMode,
    MarkXY<DOMAIN, XVAL, YVAL>
