package io.data2viz.charts.chart.mark

import io.data2viz.charts.dimension.Dimension
import io.data2viz.color.Color

/**
 * A visual element which stroke color can change depending on the data.
 */
public interface HasStrokeColor<DOMAIN> {

    /**
     * The [Dimension] used to define the stroke color from a DOMAIN object.
     */
    public var strokeColor: Dimension<DOMAIN, Color?>

    /**
     * The [Dimension] used to define the "highlighted" stroke color from a DOMAIN object.
     */
    public var strokeColorHighlight: Dimension<DOMAIN, Color?>

    /**
     * The [Dimension] used to define the "selected" stroke color from a DOMAIN object.
     */
    public var strokeColorSelect: Dimension<DOMAIN, Color?>

    @Deprecated(
        "Use strokeColor instead",
        ReplaceWith("strokeColor")
    )
    public var stroke: Dimension<DOMAIN, Color?>
        get() = strokeColor
        set(value) {strokeColor = value}

}

@Deprecated(
    "Use HasStrokeColor instead.",
    ReplaceWith("HasStrokeColor<DOMAIN>", "io.data2viz.charts.chart.mark.HasStrokeColor")
)
public typealias HasStroke<DOMAIN> = HasStrokeColor<DOMAIN>
