package io.data2viz.charts.chart.mark

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.mark.internal.HistogramMarkImpl
import io.data2viz.charts.dimension.Dimension
import io.data2viz.charts.dimension.Quantitative
import io.data2viz.charts.dimension.Temporal
import kotlinx.datetime.Instant
import kotlin.time.Duration

/**
 * Instantiate a new vertical [HistogramMark], this is similar to a [BarMark] but with a continuous quantitative X axis.
 *
 * @param x The [Quantitative] dimension for the X axis.
 * @param y The [Quantitative] dimension for the Y axis.
 * @param bandwidth The [Dimension] for computing the "thickness" of each bar, should return a Double?
 * @param init The initialization lambda for this [Mark].
 */
public fun <DOMAIN> Chart<DOMAIN>.verticalHistogram(
    x: Quantitative<DOMAIN>,
    y: Quantitative<DOMAIN>,
    bandwidth: Dimension<DOMAIN, Double?>,
    init: HistogramMark<DOMAIN, Double?, Double?, Double?>.() -> Unit = {}
): Unit =

    addMark { markIndex -> HistogramMarkImpl(markIndex, dataset, x, y, bandwidth,true, init, config.events, config.mark, config.x, config.y) }


/**
 * Instantiate a new vertical [HistogramMark], this is similar to a [BarMark] but with a continuous temporal X axis.
 *
 * @param x The [Temporal] dimension for the X axis.
 * @param y The [Quantitative] dimension for the Y axis.
 * @param bandwidth The [Dimension] for computing the "thickness" of each bar, should return a Duration.
 * @param init The initialization lambda for this [Mark].
 */
public fun <DOMAIN> Chart<DOMAIN>.verticalHistogram(
    x: Temporal<DOMAIN>,
    y: Quantitative<DOMAIN>,
    bandwidth: Dimension<DOMAIN, Duration>,
    init: HistogramMark<DOMAIN, Instant, Double?, Duration>.() -> Unit = {}
): Unit =

    addMark { markIndex -> HistogramMarkImpl(markIndex, dataset, x, y, bandwidth, true, init, config.events, config.mark, config.x, config.y) }

/**
 * Instantiate a new horizontal [HistogramMark], it is similar to a [BarMark] but with a continuous quantitative Y axis.
 *
 * @param x The [Quantitative] dimension for the X axis.
 * @param y The [Quantitative] dimension for the Y axis.
 * @param bandwidth The [Dimension] for computing the "thickness" of each bar, should return a Double?
 * @param init The initialization lambda for this [Mark].
 */
public fun <DOMAIN> Chart<DOMAIN>.horizontalHistogram(
    x: Quantitative<DOMAIN>,
    y: Quantitative<DOMAIN>,
    bandwidth: Dimension<DOMAIN, Double?>,
    init: HistogramMark<DOMAIN, Double?, Double?, Double?>.() -> Unit = {}
): Unit =

    addMark { markIndex -> HistogramMarkImpl(markIndex, dataset, x, y, bandwidth, false, init, config.events, config.mark, config.x, config.y) }


/**
 * Instantiate a new vertical [HistogramMark], this is similar to a [BarMark] but with a continuous temporal X axis.
 *
 * @param x The [Quantitative] dimension for the Y axis.
 * @param y The [Temporal] dimension for the X axis.
 * @param bandwidth The [Dimension] for computing the "thickness" of each bar, should return a Duration.
 * @param init The initialization lambda for this [Mark].
 */
public fun <DOMAIN> Chart<DOMAIN>.horizontalHistogram(
    x: Quantitative<DOMAIN>,
    y: Temporal<DOMAIN>,
    bandwidth: Dimension<DOMAIN, Duration>,
    init: HistogramMark<DOMAIN, Double?, Instant, Duration>.() -> Unit = {}): Unit =

    addMark { markIndex -> HistogramMarkImpl(markIndex, dataset, x, y, bandwidth, false, init, config.events, config.mark, config.x, config.y) }


public interface HistogramMark<DOMAIN, XVAL, YVAL, TVAL> :
    HasFill<DOMAIN>,
    HasStrokeWidth<DOMAIN>,
    HasStrokeColor<DOMAIN>,
    HasMarkDecorator<DOMAIN>,
    HasHighlightDecorator<DOMAIN>,
    HasSelectionDecorator<DOMAIN>,
    HasLegendDecorator<DOMAIN>,
    HasCustomTooltip<DOMAIN>,
    HasStack,
    HasHighlightMode,
    HasSelectionMode,
    MarkXY<DOMAIN, XVAL, YVAL> {

    /**
     * The gap is the fixed positive value in "pixels" between 2 bars.
     * Set gap to 0 (default value) for bars to stick together, set it to a higher value for better readability.
     * Note that this works in addition to the bandwidth parameter and so, should be used with care as it implies
     * to display incorrect values.
     *
     * @see bandwidth
     */
    public var gap: Int

}
