package io.data2viz.charts.chart.mark

import io.data2viz.shape.curves

/**
 * A collection of standard [PathDrawer]s.
 */
public object MarkCurves {

    /**
     * Straight line generator, draws straight lines from each point of the [Path] to the next one.
     */
    public val Straight: PathDrawer = PathDrawer { curves.linear(it) }

    /**
     * Curved line generator, draws a curve passing through each point of the [Path].
     *
     * This [PathDrawer] produces a curve through monotone cubic interpolation, which preserves monotonicity in Y.
     *
     * See [Monotone cubic interpolation](https://en.wikipedia.org/wiki/Monotone_cubic_interpolation) for more info.
     */
    public val Curved: PathDrawer = PathDrawer { curves.monotoneX(it) }

    /**
     * Step line generator, draw a line consisting of alternate horizontal and vertical lines.
     *
     * This [PathDrawer] produces a curve that changes y-value at the midpoint of each pair of adjacent x-values.
     *
     * See [Step function](https://en.wikipedia.org/wiki/Step_function) for more info.
     */
    public val Step: PathDrawer = PathDrawer { curves.step(it) }
}
