package io.data2viz.charts.chart.mark

import io.data2viz.charts.config.AxisConfig

/**
 * A [Scaler] transforms a VALUE object into a [Double], it is used to translates data into screen position.
 */
public interface Scaler<VALUE> {

    /**
     * Produces a [List] of VALUE that will be used to place "ticks" on an axis.
     *
     * To provide custom ticks, use the dimension.formatter lambda.
     */
    public val tickValues: List<VALUE>

    /**
     * Transform a VALUE object into a [Double].
     */
    public fun scaleValue(value: VALUE): Double

    /**
     * Transform a [Double] into a VALUE object (if possible), this is the inverse of the [scaleValue] function.
     *
     * @see scaleValue
     */
    public fun invert(value: Double): VALUE
}
