package io.data2viz.charts.config

import io.data2viz.charts.chart.mark.ShapeColor
import io.data2viz.charts.core.Zoom
import io.data2viz.color.Color
import io.data2viz.color.Colors
import io.data2viz.math.pct

/**
 * The [Zoom] configuration, styling information for the zooming rectangle layer, and min/max properties to define
 * "zooming limits".
 */
public data class ZoomConfig(

    /**
     * The filling [Color] for the  zooming (reversed) selection layer.
     */
    override var fill: Color? = Colors.Web.black.withAlpha(50.pct),

    /**
     * The stroke [Color] for the zooming selection layer.
     */
    override var strokeColor: Color? = null,

    /**
     * The filling thickness for the zooming selection layer.
     */
    override var strokeWidth: Double? = null,

    /**
     * The minimum [Zoom] ratio for the X axis
     */
    var minXZoomRatio: Double? = .000001,

    /**
     * The maximum [Zoom] ratio for the X axis
     */
    var maxXZoomRatio: Double? = 1000000.0,

    /**
     * The minimum [Zoom] ratio for the Y axis
     */
    var minYZoomRatio: Double? = .000001,

    /**
     * The maximum [Zoom] ratio for the Y axis
     */
    var maxYZoomRatio: Double? = 1000000.0
): ShapeColor
