package io.data2viz.charts.core

/**
 * Cursor mode: change cursor positioning behavior.
 */
public enum class CursorMode {

    /**
     * Cursor "sticks" on mark in both X and Y (default value).
     * The cursor will always be placed on the currently highlighted mark.
     *
     * @see HighlightMode
     */
    StickOnXY,

    /**
     * Cursor "sticks" on mark in X only.
     * The cursor will always be placed on the currently X-coordinate of the highlighted mark but is free on Y.
     *
     * @see HighlightMode
     */
    StickOnX,

    /**
     * Cursor "sticks" on mark in Y only.
     * The cursor will always be placed on the currently Y-coordinate of the highlighted mark but is free on X.
     *
     * @see HighlightMode
     */
    StickOnY,

    /**
     * Cursor is free on X and Y and does not "stick" to the currently highlighted mark.
     */
    Free
}
