package io.data2viz.charts.core

/**
 * Decorate a <DOMAIN> object with some information.
 *
 * @property [currentSeries] the list of Datum<DOMAIN> objects for the current object series (if a series dimension is set)
 * @property [domain] the current <DOMAIN> object
 * @property [indexInData] the index of this domain object in the initial data list
 * @property [indexInSeries] the index of this domain object in its series
 * @property [indexOfSeries] the index of this domain object series
 */
public data class Datum<DOMAIN>(
    private val dataset: Dataset<DOMAIN>,
    val domain: DOMAIN,
    val indexInData: Int
) {

    internal var _indexOfSeries: Int = 0
    val indexOfSeries: Int
        get() = _indexOfSeries

    internal var _indexInSeries: Int = indexInData
    val indexInSeries: Int
        get() = _indexInSeries


    /**
     * Get the full List of Datum this Datum belongs to.
     * If you have only one series, then currentSeries == (your dataset)
     */
    public val currentSeries: List<Datum<DOMAIN>>
        get() = dataset.dataBySeries[indexOfSeries]

    override fun hashCode(): Int {
        return indexInData
    }

    override fun equals(other: Any?): Boolean {
        return other === this
    }
//
//    /**
//     * Get the next Datum<DOMAIN> object in the series, or null if current is the last one.
//     */
//    fun nextInSeries() = if (indexOfSeries + 1 >= series.size) null else series[indexOfSeries + 1]
//
//    /**
//     * Get the previous Datum<DOMAIN> object in the series, or null if current is the first one.
//     */
//    fun previousInSeries() = if (indexOfSeries - 1 < 0) null else series[indexOfSeries - 1]

}
