package io.data2viz.charts.core

import io.data2viz.charts.config.internal.FontImpl
import io.data2viz.color.Color
import io.data2viz.color.Colors
import io.data2viz.viz.FontFamily
import io.data2viz.viz.FontPosture
import io.data2viz.viz.FontWeight


/**
 * Instantiate a [Font].
 */
public fun Font(
        fontSize: Double = 12.0,
        fontColor: Color = Colors.Web.black,
        fontFamily: FontFamily = FontFamily.SANS_SERIF,
        fontWeight: FontWeight = FontWeight.NORMAL,
        fontStyle: FontPosture = FontPosture.NORMAL
): Font = FontImpl(fontSize, fontColor, fontFamily, fontWeight, fontStyle)

/**
 * All the font properties needed to draw text.
 *
 * Fonts properties are hierarchically defined in the configuration.
 *
 * You can first define them in the root configuration (@see [ChartConfig]) and
 * then the child configuration ([AxisConfig], [LegendConfigImpl], [MarkConfig], [TooltipConfigImpl])
 * will use the same by default.
 */
public interface Font {

    /**
     * The font size.
     */
    public var fontSize: Double

    /**
     * The font [Color].
     */
    public var fontColor: Color

    /**
     * The font family ([FontFamily]).
     */
    public var fontFamily: FontFamily

    /**
     * The font weight ([FontWeight]): bold or normal.
     */
    public var fontWeight: FontWeight

    /**
     * The font posture ([FontPosture]): italic or normal.
     */
    public var fontStyle: FontPosture

}

internal fun Font.copyFont(): Font = FontImpl(fontSize, fontColor, fontFamily, fontWeight, fontStyle)


