package io.data2viz.charts.core

/**
 * External layout of a [Legend] box, where it will be placed on screen.
 */
public enum class LegendPosition {

    /**
     * (Default value) Place the [Legend] if there is enough free space, on the chart drawing zone.
     * First he legend will be placed top-left, if there is not enough free space it will be placed top-right,
     * bottom-right, then bottom-left, lastly, it won't be displayed.
     */
    Floating,

    /**
     * Place the [Legend] on the upper part of the chart.
     */
    Top,

    /**
     * Place the [Legend] on the left part of the chart.
     */
    Left,

    /**
     * Place the [Legend] on the right part of the chart.
     */
    Right,

    /**
     * Place the [Legend] on the lower part of the chart.
     */
    Bottom;

    internal val isLeftOrRight
        get() = this == Left || this == Right


    internal fun toLayout(): LayoutPosition? = when (this) {
        Top     -> LayoutPosition.Top
        Bottom  -> LayoutPosition.Bottom
        Right   -> LayoutPosition.Right
        Left    -> LayoutPosition.Left
        else    -> null
    }

}
