package io.data2viz.charts.core

import io.data2viz.charts.config.internal.StrokeImpl
import io.data2viz.color.Color

/**
 * All the stroke properties needed to draw signs.
 *
 * Stroke properties are hierarchically defined in the configuration.
 *
 * You can first define them in the root configuration (@see [ChartConfig]) and
 * then the child configuration ([AxisConfig], [LegendConfigImpl], [MarkConfig], [TooltipConfigImpl])
 * will use the same by default.
 */
public interface Stroke {

    /**
     * The stroke [Color], use null for no stroke.
     */
    public var strokeColor: Color?

    /**
     * The stroke thickness, null for no stroke.
     */
    public var strokeWidth: Double?

}

internal fun Stroke.copyStroke(): Stroke = StrokeImpl(
    strokeColor,
    strokeWidth)


