package io.data2viz.charts.core

import io.data2viz.charts.chart.mark.Mark
import io.data2viz.charts.event.EventZone
import io.data2viz.geom.Point
import io.data2viz.geom.Rect

/**
 * A [Zone] represents a simple shape that indicates this screen zone can interact with user.
 *
 * @see EventZone
 * @see Mark.getEventZones
 * @see RectZone
 * @see CircleZone
 */
public sealed interface Zone {

    /**
     * Does this [Zone] contains the given [Point]?
     *
     * @param point the coordinates relative to the main charting zone.
     * @return true if the point is in this [Zone].
     */
    public fun contains(point: Point): Boolean

    /**
     * Define an "anchor point" at the given [LayoutPosition], useful to place labels or tooltips over a [Zone].
     *
     * @param layoutPosition the position on which you want to attach an element.
     * @return a [Point] which coordinates indicate the possible "anchor".
     */
    public fun getAnchorPoint(layoutPosition: LayoutPosition): Point

    /**
     * The bounds of this [Zone] as a [Rect].
     */
    public val bounds: Rect

    /**
     * The geometric center coordinates of this [Zone].
     */
    public val center: Point
}
