package io.data2viz.charts.event

import io.data2viz.charts.chart.mark.Mark
import io.data2viz.charts.core.Datum
import io.data2viz.charts.core.TriggerMode
import io.data2viz.charts.core.Zone
import io.data2viz.geom.Point

/**
 * An [EventZone] is an on-screen zone where an event can occur related to a given [Datum].
 * The [EventZone] contains a [Zone] which is a geometric shape that is generally tied to the related sign.
 *
 * @see Zone
 * @see Mark.getEventZones
 */
public data class EventZone<DOMAIN>(

    /**
     * The [Datum] from which the sign that triggers an event originated.
     */
    val datum: Datum<DOMAIN>,

    /**
     * The interactive [Zone] in the [Mark] that triggers.
     */
    val zone: Zone,

    /**
     * The index of the mark that contains this eventZone
     */
    val markIndex: Int,

    /**
     * Give an optional [origin] to your zone, to define where the cursor and tooltip should point to.
     * Origin default value is the zone center.
     *
     * Is also used in the [TriggerMode.Nearest].
     *
     * Can be outside of [zone], useful especially when providing more than one EventZone for complex shapes.
     */
    val origin: Point = zone.center //TODO: Maybe rename to anchorPoint
)
