package io.data2viz.charts.event

import io.data2viz.geom.Point
import io.data2viz.viz.HasMetaKeys
import io.data2viz.viz.KPointer
import io.data2viz.viz.MouseButtonPressed
import io.data2viz.viz.PointerType

/**
 * Contains information from the original user event.
 *
 * @property eventType The event type
 * @property pointerType The pointer type
 * @property currentPosition The event position relative to the current charting area,
 *  in case of multiple pointers (touches), the currentPosition is the "center" of all.
 * @property buttonPressed Is a mouse button has been pressed during the event?
 * @property pointerIndex The pointer index, when multitouch, indicates the pointer index in the [pointers] list.
 * @property pointers The complete list of pointers (including the active one at pointerIndex) in case of multitouch
 */
public interface UserEvent : HasMetaKeys {
    public val eventType: EventType
    public val pointerType: PointerType
    public val currentPosition: Point
    public val buttonPressed: MouseButtonPressed
    public val pointerIndex: Int
    public val pointers: List<KPointer>
}
