package io.data2viz.charts.layout

import io.data2viz.charts.chart.Chart


/**
 * Chart layout synchronizer, allow to synchronize several charts' sizes on a vertical or horizontal layout.
 *
 * It is instantiated from a [SizeManager] through [SizeManager.hSynchro] and [SizeManager.vSynchro]
 */
public interface SizeSynchronizer {
    /**
     * Add a [Chart] to this layout synchronizer.
     */
    public fun addChart(chart: Chart<*>): Unit

    /**
     * Add several [Chart]s to this layout synchronizer.
     */
    public fun addAllCharts(vararg charts: Chart<*>): Unit


    @Deprecated("Use addChart instead.", ReplaceWith("addChart"))
    public fun addView(chart: Chart<*>): Unit = addChart(chart)

    @Deprecated("Use addAllCharts instead.", ReplaceWith("addAllCharts"))
    public fun addAllViews(vararg views: Chart<*>): Unit

}
