package io.data2viz.charts

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.chart.chart
import io.data2viz.charts.viz.VizContainer
import io.data2viz.geom.Size
import io.data2viz.viz.*



fun <DOMAIN> headlessChart(data: List<DOMAIN>,
                           init: Chart<DOMAIN>.() -> Unit
): Chart<DOMAIN> = DebugVizContainer().chart(data, init = init)


class DebugVizContainer(
    private val resizableSupport: ResizableSupport = ResizableSupport()) :
    VizContainer, Resizable by resizableSupport {


    override fun newViz(init: Viz.() -> Unit): Viz {
        val viz = Viz()
        _vizList.add(viz)
        val renderer = DebugRenderer(viz)
        viz.renderer = renderer
        return viz
    }

    override val density: Double
        get() = 1.0

    override var size: Size = Size(100.0, 100.0)


    private val _vizList = mutableListOf<Viz>()
    override val vizList: List<Viz>
        get() = _vizList
}


/**
 * Used to test and debug Charts (todo move to d2v?)
 *
 */
internal class DebugRenderer(
    override var viz: Viz
) : VizRenderer {

    override fun render() {}
    override fun startAnimations() {}
    override fun stopAnimations() {}

}


inline fun <reified N : Node> Viz.select(classes: String): List<N> {
    return selectByClasses(classes).filterIsInstance<N>()
}

fun Viz.selectByClasses(classes: String): List<Node> {
    val ret = mutableListOf<Node>()

    fun Node.select(classes: String) {
        if (classes == this.classes)
            ret.add(this)
        when (val node: Node = this) {
            is GroupNode -> node.children.forEach { child -> child.select(classes) }
            is Layer -> node.children.forEach { child -> child.select(classes) }
        }
    }
    this.layers.forEach { layer -> layer.select(classes) }
    return ret
}
